/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("dijit.Toolbar");
dojo.require("dijit.Dialog");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.FilteringTable");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.rational.report.web.core.app.view.admin.AddUsersDialog");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.rational.report.web.core.client.internal.SecurityClient");
dojo.require("com.ibm.rational.report.web.core.client.internal.UserClient");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");

var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var SecurityClient			= com.ibm.rational.report.web.core.client.internal.SecurityClient;
var UserClient				= com.ibm.rational.report.web.core.client.internal.UserClient;
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var InternalMessages		= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");
var tickSrc 				= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/tick.gif");

/**
 * Widget to display a report node.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/templates/PermissionsWidget.html"),
			
	constructor: function(args) {
		this.repositoryItemId = args.repositoryItemId;
		this.messageArea = args.messageArea;
		this._parentWidget = args.parentWidget;
		this._resourceType = args.resourceType;
	},	
		
	postCreate: function() {
		this._allPrincipals = new Object();
		this._principalMap = new Object();
		this._aclEntryPrincipals = new Array();
		this._permissionTypes = new Array();
		this._lastSelectedRow = -1;
		
		var self = this;
		
		this._permissionsChanged = false;
		this._msgFormatter = new MessageFormatter(this.messageArea);
		this._localizedPermissionNames = new Object();
		this._localizedPermissionNames["Read"] = InternalMessages.READ_PERMISSION_NAME;
		this._localizedPermissionNames["Update"] = InternalMessages.UPDATE_PERMISSION_NAME;
		this._localizedPermissionNames["Delete"] = InternalMessages.DELETE_PERMISSION_NAME;
		this._localizedPermissionNames["Execute"] = InternalMessages.EXECUTE_PERMISSION_NAME;
		this._refreshTable();
	},
	
	_refreshTable: function(){
		// gets all security principals
		var srh = new ServiceResponseHandler(this, "_getAllPrincipals",	"_getAllUsersError");
		var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();				
		var params = { projectAreaId: projectArea.itemId };				
		UserClient.getAllUsers(srh);
	},
	_getAllPrincipals: function(principals) {
		Util.removeChildren(this.messageArea);
		if (principals != null) {
			var sortedPrincipals = Util.sort(principals);
			for (var i = 0; i < sortedPrincipals.length; i++) {
				var principal = sortedPrincipals[i];
				this._principalMap[principal.name] = principal;
			}
			this._allPrincipals = sortedPrincipals;
		}
		
		if (this.repositoryItemId != null) {
			var srh = new ServiceResponseHandler(this, "_getAclEntriesForResource", "_getAclEntriesError");
			var params = {type: this._resourceType, repositoryItemId: this.repositoryItemId, principalId: ""};
			SecurityClient.getAclEntries(srh, params);
		}
		else {
			this._getAclEntriesForResource(null);
		}
	},
	
	_getAllUsersError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETRIEVE_USERS_AND_TEAMS, error);
	},
	
	_getAclEntriesForResource: function(aclEntries) {
		if (aclEntries != null) {
			var principals = new Array();
			var index = 0;
			for (var i = 0; i < aclEntries.length; i++) {
				var aclEntry = aclEntries[i];
				var principal = this._principalMap[aclEntry.principalId];
				if (typeof principal != "undefined" && principal != null) {
					principals[index] = principal;
					principals[index]._aclEntry = aclEntry;
					index++;
				}
			}
			var sortedPrincipals = Util.sort(principals);
			this._aclEntryPrincipals = principals;
		}
		
		var srh = new ServiceResponseHandler(this, "_getPermissionsForResource", "_getPermissionsError");
		if (this._resourceType == "REPORT_FOLDER") {
			SecurityClient.getReportFolderPermissions(srh);
		}
		
		else if (this._resourceType == "REPORT_DESIGN") {
			SecurityClient.getReportDesignPermissions(srh);
		}
		else if (this._resourceType == "REPORT_QUERY") {
			SecurityClient.getReportQueryPermissions(srh);
		}
		else if (this._resourceType == "REPORT_SNAPSHOT") {
			SecurityClient.getReportSnapshotPermissions(srh);
		}
		else if (this._resourceType == "DATA_SOURCE") {
			SecurityClient.getDataSourcePermissions(srh);
		}				
	},
	
	_getAclEntriesError: function(error) {
		var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_RETRIEVE_ACL_ENTRIES, [dojo.trim(this.nodeName.value)]);
		this._msgFormatter.displayErrorMessage(msg, error);
	},
	
	_getPermissionsForResource: function(permissions) {
		Util.removeChildren(this.messageArea);
		if (permissions != null) {
			this._permissionTypes = permissions;
		}	
		//now all data is available for the table to be constructed
		this._createTable();
	},
	
	_getPermissionsError: function(error) {
		var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_RETRIEVE_PERMISSIONS, [dojo.trim(this.nodeName.value)]);
		this._msgFormatter.displayErrorMessage(msg, error);
	},
	
	_createTable: function() {
		if (this._table == null) {
			var chk = document.createElement("input");
			chk.type = "checkbox";
			
			
			var toolbar = new dijit.Toolbar({
				layoutAlign: "top",
				style: "float: left; background : transparent; border: none"
			});
			toolbar.addChild(new dijit.form.Button({
				title: CommonMessages.ADD_USERS_TOOLTIP,
				iconClass: "AddButtonIcon",
				label: CommonMessages.ADD_USERS_TOOLTIP,
				showLabel: false,
				onClick: dojo.hitch(this, this._addUsers)
			}));
			toolbar.addChild(new dijit.form.Button({
				title: CommonMessages.DELETE_USERS_TOOLTIP,
				iconClass: "RemoveButtonIcon",
				label: CommonMessages.DELETE_USERS_TOOLTIP,
				showLabel: false,
				onClick: dojo.hitch(this, this._deleteUsers)
			}));
			
			this._tableColumns = [{
				field: "Select",
				sortType: "__markup__",
				label: chk,
				width: "12px"
			}, {
				field: "Name",
				label: InternalMessages.NAME
			}];
			
			for (var i = 0; i < this._permissionTypes.length; i++) {
				this._tableColumns.push({
					field: this._permissionTypes[i].name,
					sortType: "__markup__",
					noSort: true,
					label: this._localizedPermissionNames[this._permissionTypes[i].name],
					width: "50px"		
				});
			}
			
			this._tableColumns.push({
				field: "Toolbar",
				sortType: "__markup__",
				label: toolbar.domNode
			});
			
			var sortInformation = [{
				index: 1,
				direction: 0
			}];			
			this._table = new com.ibm.rational.report.web.core.app.internal.common.FilteringTable({
				valueField: "rowId",
				noSelectField: "Select",
				sortInformation: sortInformation,
				selectOnHover: true
			}, this.permissionsTable);
			for (var i = 0; i < this._tableColumns.length; i++) {
				this._table.columns.push(this._table.createMetaData(this._tableColumns[i]));
			}
			
			dojo.connect(this._table, "onSelect", this, "_onTableSelect");
			dojo.connect(this._table, "onHover", this, "_onTableHover");
		}
		this._displayPermissions();
	},
	
	_onTableSelect: function() {
		if (this._table.isColumnSelected(0)) {
			return;
		}
		var data = this._table.getSelectedData();
		var selectedRow = data.rowId;
		
		if (selectedRow == this._lastSelectedRow) {
			this._permissionsChanged = true;
			if (this._parentWidget != null) {
				this._parentWidget._updateSaveButtonState();
			}
			
			//save the permissions in aclEntryPrincipals
			var principalName = data["Name"];
			var permissions = new Array();
			for(var i = 0;i<this._permissionTypes.length;i++) {
				var chk = data[this._permissionTypes[i].name];
				if(chk.checked) {
					permissions.push(this._permissionTypes[i]);
				}
			}
			for(var i=0;i<this._aclEntryPrincipals.length;i++) {
				if(this._aclEntryPrincipals[i].name == principalName){
					this._aclEntryPrincipals[i]._aclEntry.permissions = permissions;
					this._aclEntryPrincipals[i]._aclEntry.modified = true;
					break;
				}
			}
		}
		this._lastSelectedRow = data.rowId;
	},
	
	_onTableHover: function() {
		if (this._table.isColumnSelected(0)) {
			return;
		}
		var data = this._table.getSelectedData();
		var selectedRow = data.rowId;
		
		if(selectedRow != this._lastSelectedRow) {
			//restore last selected row
			if (this._lastSelectedRow >= 0) {			
				var prevData = this._table.store.getDataByKey(this._lastSelectedRow);
				for (var i = 0; i < this._permissionTypes.length; i++) {
					var tick = document.createElement("img");
					tick.src = tickSrc;
					
					var chk = prevData[this._permissionTypes[i].name];
					var image = (chk.checked) ? tick : null;
					this._table.store.update(prevData,this._permissionTypes[i].name,image);
				}
			}
			//update current selected row
			for (var i = 0; i < this._permissionTypes.length; i++) {
				var chk = document.createElement("input");
				chk.type = "checkbox";
				
				var img = data[this._permissionTypes[i].name];
				chk.checked = (img!=null) ? true : false;
				this._table.store.update(data,this._permissionTypes[i].name, chk);
			}
		}
		this._lastSelectedRow = data.rowId;
	},
	
	_displayPermissions: function() {
		Util.removeChildren(this.messageArea);
		this._table.store.clearData();
		this._lastSelectedRow = -1;
		if (this._aclEntryPrincipals) {
			for (var i = 0; i < this._aclEntryPrincipals.length; i++) {
				var chk = document.createElement("input");
				chk.type = "checkbox";
				chk.id = this._aclEntryPrincipals[i].name;			
				chk.title = CommonMessages.SELECT_USER_TOOLTIP;		
				var data = new Object();
				data["rowId"] = i;
				data["Select"] = chk;
				data["Name"] = this._aclEntryPrincipals[i].name;
				data["Toolbar"] = null;

				var aclEntry = this._aclEntryPrincipals[i]._aclEntry;
				var permissions = aclEntry.permissions;
				var negative = aclEntry.negative;
				
				var aclPermissions = new Object();
				for (var k = 0; k < permissions.length; k++) {
					var permission = permissions[k];
					aclPermissions[permission.itemId] = permission;
				}
				//for all permissionTypes
				for (var j = 0; j < this._permissionTypes.length; j++) {
					var aclPermission = aclPermissions[this._permissionTypes[j].itemId];
					var per;
					if (typeof aclPermission != "undefined" && aclPermission != null) {
						per = (negative) ? false : true;
					}
					else {
						per = false;
					}
					var tick = document.createElement("img");
					tick.src = tickSrc;
					var image = (per) ? tick : null;
					data[this._permissionTypes[j].name] = image;
				}
				this._table.store.addData(data, i);

			}
		}
	},
	
	_updateAclEntries: function() {
		// Save the permission settings of the ACL entries		
		for(var i=0;i<this._aclEntryPrincipals.length;i++) {
			var permissionIds = new Array();
			var aclEntry = this._aclEntryPrincipals[i]._aclEntry;
			if(aclEntry.modified) {
				var permissions = aclEntry.permissions;
				for (var j = 0; j < permissions.length; j++) {				
					var permission = permissions[j];
					permissionIds[j] = permission.itemId;
				}

				var srh = new ServiceResponseHandler(this, "_postUpdateAclEntrySuccess", "_postUpdateAclEntryError");
				var params = {id: aclEntry.itemId, name: "", repositoryItemId: "",
							  principalId: "", permissionIds: permissionIds, negative: false};
				SecurityClient.postUpdateAclEntry(srh, params);
			}
		}
		this._permissionsChanged = false;
		this._displayPermissions();
	},
	
	_postUpdateAclEntrySuccess: function() {
		Util.removeChildren(this.messageArea);
	},
	
	_postUpdateAclEntryError: function(error) {
		var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_UPDATE_ACL_ENTRIES, [dojo.trim(this.nodeName.value)]);
		this._msgFormatter.displayErrorMessage(msg, error);
	},
		
	_addUsers : function() {
		var resource = new Object();
		resource.itemId = this.repositoryItemId;
		resource.type = this._resourceType;
		
		var self = this;
		var addUsersDialog = new com.ibm.rational.report.web.core.app.view.report.AddUsersDialog({
			controller: this._controller,
			title: CommonMessages.ADD_USERS_DIALOG_TITLE,
			resource: resource,
			principalMap: this._principalMap,
			allPrincipals: this._allPrincipals,
			aclEntryPrincipals: this._aclEntryPrincipals,
			postSave: this._postSave,
			postCancel: this._postCancel,
			parentWidget: this
		});
		dojo.connect(addUsersDialog.okButton, "onclick", addUsersDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				addUsersDialog.addContributers();
				addUsersDialog.close();
			}
		);	
		dojo.connect(addUsersDialog.cancelButton, "onclick", addUsersDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				addUsersDialog.close();
			}
		);	
	},
	
	_deleteUsers: function() {
		var inputs = [];
		var rows = this.permissionsTable.tBodies[0].rows;					  	
        for (var i = 0; i < rows.length; i++) {
        	var td = rows[i].cells[0];
            if (td.firstChild) {
	            inputs.push(td.firstChild);
            }
        }		
		
		var removedPrincipals = new Array();
		dojo.forEach(inputs, function(input) {
			if (input.checked) {			
				removedPrincipals.push(input.id);
			}
		});		
		if (removedPrincipals != null && removedPrincipals.length > 0) {
			var aclEntryIds = new Array();
			for(var j=0 ; j<this._aclEntryPrincipals.length ; j++){
				for(var k=0 ; k<removedPrincipals.length; k++) {
					if(this._aclEntryPrincipals[j].name == removedPrincipals[k]){
						aclEntryIds.push(this._aclEntryPrincipals[j]._aclEntry.itemId);
						break;
					}
				}
			}
			var srh = new ServiceResponseHandler(this, "_postDeleteAclEntriesSuccess", "_postDeleteAclEntriesError");
			SecurityClient.postDeleteAclEntries(srh, {ids: aclEntryIds});
		}
		else {
			this._postDeleteAclEntriesSuccess();
		}
		
	},
	
	_postDeleteAclEntriesSuccess: function() {
		var inputs = this.permissionsTable.tBodies[0].getElementsByTagName("input");
		var removedPrincipals = new Array();
		dojo.forEach(inputs, function(input) {
			if (input.checked) {			
				removedPrincipals.push(input.id);
			}
		});	
		var newAclEntryPrincipals = new Array();
		if (removedPrincipals != null && removedPrincipals.length > 0) {
			for(var i = 0;i<this._aclEntryPrincipals.length;i++) {
				var removed = false;
				for(var j=0;j<removedPrincipals.length;j++){
					if(removedPrincipals[j] == this._aclEntryPrincipals[i].name) {
						removed = true;
						break;
					}
				}
				if(!removed) {
					newAclEntryPrincipals.push(this._aclEntryPrincipals[i]);
				}
			}
			this._aclEntryPrincipals = newAclEntryPrincipals;
			this._displayPermissions();
		}		
	},
	
	_postDeleteAclEntriesError: function() {
		//display error message
	}
});

})();